;===========================================================================================
; TimeWinColor.asm  Fasm Flat assembler
; Temps de travail de Windows en couleur
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

  WSCREEN = 380
  HSCREEN = 42
  GREEN   = 0FF00h
  BLACK   = 0h

;===========================================================================================
; Dbut du code programme
;-------------------------------------------------------------------------------------------
section '.code' code readable executable

start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne
    ; ----------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur
    ; -----------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Valider le choix de la structure WNDCLASS
    ; -----------------------------------------
    invoke RegisterClass,wc

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,WSCREEN
    shr eax,1
    mov [rect.left],eax

    ; Ajuster les coordonnes de la fentre
    ; -------------------------------------
    mov [rect.top],0
    mov [rect.right],WSCREEN
    mov [rect.bottom],HSCREEN
    invoke AdjustWindowRectEx,rect,WS_CAPTION+WS_VISIBLE+WS_SYSMENU+WS_MINIMIZEBOX,\
                                                                            0,NULL
    mov eax,[rect.bottom]
    sub eax,3                ;hauteur bord infrieur fentre active
    sub eax,[rect.top]
    sub eax,HSCREEN
    inc eax                  ;hauteur bord infrieur bandeau titre
    mov [HBarTitle],eax      ;hauteur totale du bandeau titre

    invoke GetSystemMetrics,SM_CYSCREEN
    sub eax,HSCREEN
    sub eax,[HBarTitle]
    shr eax,1
    mov [rect.top],100       ;pour forer Y remplacer eax par la valeur dsire

    mov eax,HSCREEN
    add eax,[HBarTitle]

    ; Cration fentre principale avec barre de titre suprieure
    ; ----------------------------------------------------------
    invoke CreateWindowEx,NULL,ClassName,WinTitreFr,\
                          WS_CAPTION+WS_VISIBLE+WS_SYSMENU+WS_MINIMIZEBOX,\
                          [rect.left],[rect.top],WSCREEN+6,eax,NULL,0,[wc.hInstance],NULL
    mov [hWnd],eax

    ; Affichage de la fentre
    ; -----------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT

    ; Rafraichissement de la fentre
    ; ------------------------------
    invoke UpdateWindow,[hWnd]

 ; Boucle de traitement des messages
 ; ---------------------------------
 MessageLoop:
    ; Rceptionne les messages
    ; ------------------------
    invoke GetMessage,msg,0,0,0
    or eax,eax
    jz ExitProgram

    ; Traduit les touches d'entre clavier en messages
    ; ------------------------------------------------
    invoke TranslateMessage,msg

    ; Expdition des messages  la procdure Windows
    ; ----------------------------------------------
    invoke DispatchMessage,msg

    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE

        ; Dtruire le timer
        ; -----------------
        invoke KillTimer,[@hWnd],1000

        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgPub,ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande la fin de l'application
        ; -------------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE

        ; Cration d'une fentre statique pour le texte
        ; ---------------------------------------------
        invoke CreateWindowEx,WS_EX_DLGMODALFRAME,StaticClass,NULL,\
                                     WS_CHILD+WS_VISIBLE+ES_CENTER,\
                             8,8,364,24,[@hWnd],0,[wc.hInstance],0
        mov [hStatic],eax

        ; Initialiser un timer
        ; --------------------
        invoke SetTimer,[@hWnd],1000,100,NULL

    .elseif [@uMsg] = WM_TIMER
        invoke GetTickCount
        mov edx,0  
        mov ecx,3600000 
        div ecx 
        mov [Heures],eax
        mov eax,edx
        mov edx,0 
        mov ecx,60000  
        div ecx
        mov [Minutes],eax
        mov eax,edx 
        mov edx,0 
        mov ecx,1000 
        div ecx 
        mov [Secondes],eax

        ; Calcul pour les heures
        ; ----------------------
        .if [Heures] < 10

            ; Convertion valeur dans une chaine
            ; ---------------------------------
            cinvoke wsprintf,HeuresBuf,sFormat,[Heures]
         
            mov edi,NullTxt
            inc edi
            mov esi,HeuresBuf
            lodsb 
            stosb   
            mov esi,NullTxt
            mov edi,HeuresBuf
            mov ecx,2 
            rep movsb 

        .else 
            cinvoke wvsprintf,HeuresBuf,sFormat,[Heures]
            
        .endif

        ; Calcul des minutes
        ; ------------------
        .if [Minutes] < 10
            cinvoke wsprintf,MinutesBuf,sFormat,[Minutes]
            
            mov edi,NullTxt
            inc edi
            mov esi,MinutesBuf
            lodsb
            stosb
            mov esi,NullTxt
            mov edi,MinutesBuf
            mov ecx,2
            rep movsb
            
        .else
            cinvoke wsprintf,MinutesBuf,sFormat,[Minutes]
            
        .endif

        ; Calcul des secondes
        ; -------------------
        .if [Secondes] < 10
            cinvoke wsprintf,SecondesBuf,sFormat,[Secondes]
            
            mov edi,NullTxt
            inc edi
            mov esi,SecondesBuf
            lodsb
            stosb
            mov esi,NullTxt
            mov edi,SecondesBuf
            mov ecx,2
            rep movsb
            
        .else
            cinvoke wsprintf,SecondesBuf,sFormat,[Secondes]
            
        .endif

        ; Concat des chanes
        ; ------------------
        cinvoke wsprintf,ResultBuf,ExpTxtFr,HeuresBuf,MinutesBuf,SecondesBuf

        ; Texte complet dans la fentre statique
        ; --------------------------------------
        invoke SetWindowText,[hStatic],ResultBuf

    ; Actualiser la fentre statique
    ; ------------------------------
    .elseif [@uMsg] = WM_CTLCOLORSTATIC

        ; Couleur du texte
        ; ----------------
        invoke SetTextColor,[@wParam],GREEN
        ; Couleur du fond du texte
        ; ------------------------
        invoke SetBkColor,[@wParam],BLACK
        ; Couleur du fond de la fentre
        ; -----------------------------
        invoke GetStockObject,BLACK_BRUSH
        ret

    .elseif [@uMsg] = WM_PAINT

        ; Dbut criture/dessin
        ; ---------------------
        invoke BeginPaint,[@hWnd],ps

        ; Fin criture/dessin
        ; -------------------
        invoke EndPaint,[@hWnd],ps

    .else
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret

endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  ClassName      db ' AsmGges Win32',0
  WinTitreFr     db ' Temps et Windows  ',0
  MsgPub         db ' AsmGges France @2006  ',0

  sFormat        db '%u',0
  StaticClass    db 'STATIC',0
  ExpTxtFr       db ' Windows travaille depuis:  %sh %smn %ss',0
  NullTxt        db '0',0,0
  align 4

  hWnd           dd 0
  HBarTitle      dd 0

  Heures         dd 0
  Minutes        dd 0
  Secondes       dd 0
  hStatic        dd 0

  ResultBuf      rb 100
  HeuresBuf      rb 10
  align 4
  MinutesBuf     rb 10
  align 4
  SecondesBuf    rb 10
  align 4

  wc             WNDCLASS  0,WindowProc,0,0,NULL,NULL,NULL,2,NULL,ClassName
  msg            MSG
  ps             PAINTSTRUCT
  rect           RECT

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons

  IDI_FRANCE = 17

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

;===========================================================================================
